@@ -1,5 +1,5 @@
 \donttest{
-if (bru_safe_inla()) {
+if (bru_safe_inla() && require(ggplot2, quietly = TRUE)) {
 
   # Load Gorilla data
   data("gorillas", package = "inlabru")
@@ -31,4 +31,4 @@ if (bru_safe_inla()) {
     gg(gorillas$mesh, color = (xcoord - 580), mask = gorillas$boundary) +
     gg(gorillas$boundary)
 }
-}
\ No newline at end of file
+}
