@@ -3,7 +3,7 @@ local_bru_testthat_setup()
 test_that("Mexdolphin: Hazard rate detection function", {
   skip_on_cran()
   local_bru_safe_inla()
-  data(mexdolphin, package = "inlabru")
+  data(mexdolphin, package = "inlabru", envir = environment())
 
   hr <- function(distance, lsig) {
     1 - exp(-(distance / (exp(lsig)))^-1)
@@ -19,9 +19,9 @@ test_that("Mexdolphin: Hazard rate detection function", {
     ips = ips,
     formula = form,
     options = list(
-      bru_verbose = 0,
+      bru_verbose = 2,
       control.inla = list(int.strategy = "auto"),
-      bru_initial = list(lsig = -1)
+      bru_initial = list(lsig = -1) # A difficult starting point
     )
   )
 
@@ -37,28 +37,26 @@ test_that("Mexdolphin: Hazard rate detection function", {
   #
   #  plot(ips$distance, hr(ips$distance, fit$summary.fixed["lsig", "mean"]))
 
-  expect_equal(fit$summary.fixed["lsig", "mean"], 1.038281, tolerance = midtol)
+  expect_equal(fit$summary.fixed["lsig", "mean"], 1.06, tolerance = midtol)
   expect_equal(fit$summary.fixed["lsig", "sd"], 0.5183252, tolerance = midtol)
-  expect_equal(fit$summary.fixed["Intercept", "mean"], 2.325408, tolerance = midtol)
+  expect_equal(fit$summary.fixed["Intercept", "mean"], 2.29, tolerance = midtol)
   expect_equal(fit$summary.fixed["Intercept", "sd"], 0.2900139, tolerance = midtol)
 })
 
 
 # timings <- function() {
+#   data(mexdolphin, package = "inlabru", envir = environment())
+#
+#   hr <- function(distance, lsig) {
+#     1 - exp(-(distance / (exp(lsig)))^-1)
+#   }
+#   cmp <- ~ lsig(1) + Intercept(1)
+#   form <- distance ~ log(hr(distance, lsig)) + Intercept
+#   ips <- ipoints(INLA::inla.mesh.1d(seq(0, 8, by = 0.1)), name = "distance")
 #   local_bru_options_set(bru_verbose = FALSE)
 #   bench::mark(
-#     legacy = {
-#       local_bru_options_set(bru_linearisation_method = "legacy")
-#       fit <- lgcp(
-#         components = cmp,
-#         mexdolphin$points,
-#         ips = ips,
-#         formula = form,
-#         options = list(control.inla = list(int.strategy = "auto"))
-#       )
-#     },
 #     pandemic = {
-#       local_bru_options_set(bru_linearisation_method = "pandemic")
+#       local_bru_options_set(bru_method = list(taylor = "pandemic"))
 #       fit <- lgcp(
 #         components = cmp,
 #         mexdolphin$points,
