@@ -5,13 +5,11 @@ test_that("2D LGCP fitting and prediction: Plot sampling", {
   local_bru_safe_inla()
 
   options <- list(
-    bru_verbose = TRUE,
-    verbose = TRUE,
     control.inla = list(
       int.strategy = "eb"
     )
   )
-  data(gorillas, package = "inlabru")
+  data(gorillas, package = "inlabru", envir = environment())
 
   matern <- INLA::inla.spde2.pcmatern(gorillas$mesh,
     prior.sigma = c(0.1, 0.01),
@@ -28,27 +26,18 @@ test_that("2D LGCP fitting and prediction: Plot sampling", {
 
   expect_equal(
     sum(fit$bru_info$lhoods[[1]]$E),
-    7.095665,
-    tolerance = midtol
+    7.096605,
+    tolerance = lowtol
   )
-  expect_equal(
-    fit$summary.fixed["Intercept", "mean"],
-    1.796279,
-    tolerance = midtol
+  # Test points selected with sd+sd less than 1.2, for a more stable check.
+  expect_snapshot_value(
+    fit$summary.fixed["Intercept", "mean"] + fit$summary.random$my.spde$mean[c(19, 100, 212)],
+    tolerance = midtol,
+    style = "serialize"
   )
-  expect_equal(
-    fit$summary.fixed["Intercept", "sd"],
-    0.5221979,
-    tolerance = midtol
-  )
-  expect_equal(
-    fit$summary.random$my.spde$mean[c(1, 100, 300)],
-    c(-1.566168, 1.177564, -1.584715),
-    tolerance = midtol
-  )
-  expect_equal(
-    fit$summary.random$my.spde$sd[c(1, 100, 300)],
-    c(1.3523279, 0.6516389, 1.2547961),
-    tolerance = midtol
+  expect_snapshot_value(
+    fit$summary.fixed["Intercept", "sd"] + fit$summary.random$my.spde$sd[c(19, 100, 212)],
+    tolerance = hitol,
+    style = "serialize"
   )
 })
